<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioOrderPrefixer.php';

class MontonioFinancing
{
    const MIN_ORDER_TOTAL = '90';
    const MAX_ORDER_TOTAL = '14999';
    const METHOD_CODE = 'MontonioFinancing';

    public static function getConfig()
    {
        return array(
            array(
                'type'    => 'switch',
                'label'   => 'Enable Montonio Financing',
                'hint'    => 'When enabled, show Montonio Financing as a payment option at checkout',
                'is_bool' => true,
                'name'    => 'MONTONIO_FINANCING_ENABLED',
                'values'  => array(
                    array(
                        'id'    => 'active_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'active_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'    => 'switch',
                'label'   => 'Show logo?',
                'hint'    => 'You can turn this off if you don\'t want to show logo at checkout',
                'is_bool' => true,
                'name'    => 'MONTONIO_FINANCING_SHOW_LOGO',
                'values'  => array(
                    array(
                        'id'    => 'financing_logo_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'financing_logo_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'     => 'text',
                'label'    => 'Min. order amount',
                'hint'     => 'The minimum order amount required for Montonio in checkout',
                'name'     => 'MONTONIO_FINANCING_MIN_ORDER_AMOUNT',
                'size'     => 10,
                'required' => true,
            ),
            array(
                'type'     => 'text',
                'label'    => 'Max. order amount',
                'desc'     => 'The recommended minimum amount is 90€',
                'hint'     => 'The maximum order amount allowed for Montonio in checkout',
                'name'     => 'MONTONIO_FINANCING_MAX_ORDER_AMOUNT',
                'size'     => 10,
                'required' => true,
            ),
            array(
                'type'    => 'select',
                'label'   => 'Enable Loan Calculator',
                'desc'    => 'Show loan calculator at product page?',
                'name'    => 'MONTONIO_FINANCING_CALC_ENABLED',
                'options' => array(
                    'query' => array(
                        array(
                            'id'   => 'yes',
                            'name' => 'Yes',
                        ),
                        array(
                            'id'   => 'no',
                            'name' => 'No',
                        ),
                    ),
                    'id'    => 'id',
                    'name'  => 'name',
                ),
            ),
            array(
                'type'     => 'text',
                'label'    => 'Info page URL',
                'hint'     => 'The hire purchase info page URL.',
                'name'     => 'MONTONIO_FINANCING_CALC_INFO_PAGE_URL',
                'size'     => 10,
                'required' => false,
            ),
            array(
                'type'    => 'select',
                'label'   => 'Show modal?',
                'desc'    => 'Show popup with instructions how to apply for financing?',
                'name'    => 'MONTONIO_FINANCING_CALC_SHOW_MODAL',
                'options' => array(
                    'query' => array(
                        array(
                            'id'   => 'yes',
                            'name' => 'Yes',
                        ),
                        array(
                            'id'   => 'no',
                            'name' => 'No',
                        ),
                    ),
                    'id'    => 'id',
                    'name'  => 'name',
                ),
            ),
            array(
                'type'         => 'textarea',
                'label'        => 'Custom CSS for Calculator',
                'name'         => 'MONTONIO_FINANCING_CALC_CSS',
                'autoload_rte' => false,
            ),
        );
    }

    const CONFIGDEFAULTS = array(
        'MONTONIO_FINANCING_ENABLED'            => '0',
        'MONTONIO_FINANCING_CREATE_ORDER'       => 'before',
        'MONTONIO_FINANCING_SHOW_LOGO'          => '1',
        'MONTONIO_FINANCING_LOGO_URL'           => 'https://public.montonio.com/logo/financing/en-financing.png',
        'MONTONIO_FINANCING_ACCESS_KEY'         => '',
        'MONTONIO_FINANCING_SECRET_KEY'         => '',
        'MONTONIO_FINANCING_ENVIRONMENT'        => '',
        'MONTONIO_FINANCING_MIN_ORDER_AMOUNT'   => self::MIN_ORDER_TOTAL,
        'MONTONIO_FINANCING_MAX_ORDER_AMOUNT'   => self::MAX_ORDER_TOTAL,
        'MONTONIO_FINANCING_CALC_ENABLED'       => '',
        'MONTONIO_FINANCING_CALC_INFO_PAGE_URL' => '',
        'MONTONIO_FINANCING_CALC_CSS'           => '',
        'MONTONIO_FINANCING_CALC_SHOW_MODAL'    => 1,
    );

    const TRANSLATIONS = array(
        'title'       => 'Montonio Financing',
        'description' => 'Compare financing offers from multiple lenders in real time.'
        . ' Choose the offer you like best and complete the purchase in a breeze.',
    );

    public static function isEnabled()
    {
        return Configuration::get('MONTONIO_FINANCING_ENABLED') === '1';
    }

    public static function getCheckoutPaymentOption($module)
    {
        /**
         * Check if option is enabled and supports selected currency
         */
        if (
            !self::isEnabled() ||
            !$module->isCurrencySupported(self::METHOD_CODE)
        ) {
            return null;
        }

        $context = Context::getContext();
        $cart = $context->cart;

        /**
         * Check if cart total is between the configured min and max
         */
        $minCartTotal = Configuration::get('MONTONIO_FINANCING_MIN_ORDER_AMOUNT');
        $maxCartTotal = Configuration::get('MONTONIO_FINANCING_MAX_ORDER_AMOUNT');

        $minCartTotal = is_numeric($minCartTotal) ? (float)$minCartTotal : (float)self::MIN_ORDER_TOTAL;
        $maxCartTotal = is_numeric($maxCartTotal) ? (float)$maxCartTotal : (float)self::MAX_ORDER_TOTAL;

        $total = $cart->getOrderTotal();
        $total = is_numeric($total) ? (float) $total : null;
        if (is_float($total) && ($total < $minCartTotal || $total > $maxCartTotal)) {
            return null;
        }

        /**
         * Form action URL. The form data will be sent to the
         * validation controller when the user presses Place Order
         */
        $formAction = $context->link->getModuleLink(
            $module->name,
            'validation',
            array('method' => self::METHOD_CODE),
            true
        );

        /**
         * Assign the url form action to the template var $action
         */
        $context->smarty->assign(array('action' => $formAction));

        $title       = $module->montonioFinancingTitle;
        $description = $module->montonioFinancingDesc;

        /**
         *  Load form template to be displayed in the checkout step
         */
        $paymentForm = $module->fetch('module:montonio/views/templates/hook/payment_options.tpl');

        /**
         * Create a PaymentOption object containing the necessary data
         * to display this module in the checkout
         */
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($module->displayName)
            ->setCallToActionText($title)
            ->setAdditionalInformation($description)
            ->setAction($formAction)
            ->setForm($paymentForm);

        /**
         * Get Logo for Payment method and
         */
        $showLogo = Configuration::get('MONTONIO_FINANCING_SHOW_LOGO');
        if ($showLogo == '1') {
            $newOption->setLogo(self::getLogoUrl());
        }

        return $newOption;
    }

    /**
     * Prepare the data to be submitted to Montonio Financing
     *
     * @param object $context - The execution context
     * @param object $module - Montonio class instance
     */
    public static function preparePaymentData($context, $module)
    {
        $cart     = $context->cart;
        $address  = new Address($cart->id_address_invoice);
        $customer = $context->customer;

        // TODO: In the future, implement Merchant Name for financing as well, see MontonioPayments.php
        $storePrefix = Configuration::get('MONTONIO_ORDER_PREFIX');

        // add Prefix to Cart ID if needed
        $cartId = (string) $cart->id;
        $cartId = MontonioOrderPrefixer::addPrefix($storePrefix, $cartId);

        $notificationUrlArgs = array(
            'method'          => self::METHOD_CODE,
            'id_cart'         => $cartId,
            'id_module'       => $module->id,
            'key'             => $customer->secure_key,
            'is_notification' => 'yes',
        );

        $returnUrlArgs = array(
            'method'    => self::METHOD_CODE,
            'id_cart'   => $cartId,
            'id_module' => $module->id,
            'key'       => $customer->secure_key,
        );

        // Shipping modules' Integration ********************************************************
        $carrier = new Carrier($cart->id_carrier);
        $carrierInfo = null;
        $carrierName = $carrier->external_module_name;
        if ($carrierName) {
            $key = $carrierName . '_' . $cart->id_address_delivery;
            if (isset($context->cookie->$key)) {
                $carrierInfo = $context->cookie->$key;
            }
        }

        if ($carrierInfo) {
            $notificationUrlArgs['extra_data'] = array($key => $carrierInfo);
            $returnUrlArgs['extra_data'] = array($key => $carrierInfo);
        }
        // /Shipping modules' Integration *******************************************************

        $moduleName = $module->name;

        /**
         * Generate order before redirect
         */
        $createOrderBefore = true;
        if (Configuration::get('MONTONIO_FINANCING_CREATE_ORDER') === 'before') {
            $createOrderBefore = true;
        };
        if($createOrderBefore) {
            $module->validateOrder(
                (int)$cart->id,
                Configuration::get('PS_OS_MONTONIO'),
                $cart->getOrderTotal(),
                $module->displayName . ' (' .self::METHOD_CODE . ', Cart ID: ' . (int)$cart->id . ')',
                'Order Created with ' . self::METHOD_CODE,
                NULL,
                (int)$context->currency->id,
                false,
                $context->customer->secure_key
            );
        }
        if (isset($module->currentOrder)) {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $module->currentOrder);
            $order = new Order((int)$module->currentOrder);
        } else {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $cartId);
            $order = $context->cart;
        }
        $merchantRef = $orderRef. ((Configuration::get('MONTONIO_ENVIRONMENT') === 'sandbox') ? '_' . rand() : '');
        $orderTotal = (isset($order->total_paid)) ? $order->total_paid : $order->getOrderTotal();

        //can change on sandbox
        $returnUrlArgs['id_cart'] = $merchantRef;
        $notificationUrlArgs['id_cart'] = $merchantRef;

        $data       = array(
            "origin"               => "online",
            "merchant_reference"   => $merchantRef,
            "customer_first_name"  => (string) $customer->firstname,
            "customer_last_name"   => (string) $customer->lastname,
            "customer_email"       => (string) $customer->email,
            "customer_city"        => (string) $address->city,
            "customer_address"     => (string) $address->address1,
            "customer_postal_code" => (string) $address->postcode,
            "products"             => array(),
            "callback_url"         => $context->link->getModuleLink($moduleName, 'callback', $returnUrlArgs),
            "notification_url"     => $context->link->getModuleLink($moduleName, 'callback', $notificationUrlArgs)
        );

        /**
         * Get products from cart and add them to data
         */
        $products = $cart->getProducts(true);

        foreach ($products as $product) {
            $data['products'][] = array(
                'quantity'      => (int) $product['quantity'],
                'product_name'  => Product::getProductName($product['id_product']),
                'product_price' => 0,
            );
        }

        /**
         * Add shipping price if needed
         * @param 5 - Only Shipping
         */
        if(isset($module->currentOrder)) {
            $shippingTotal = $order->total_shipping;
        } else {
            $shippingTotal = $order->getOrderTotal(true, 5);
        }

        if ($shippingTotal > 0) {
            $data['products'][] = array(
                'quantity'      => 1,
                'product_name'  => 'SHIPPING',
                'product_price' => 0,
            );
        }

        /**
         * Add Grand total to paymentData
         */
        $data['products'][] = array(
            'quantity'      => 1,
            'product_name'  => (string) 'Cart/Order no: ' . $merchantRef,
            'product_price' => (float) $orderTotal,
        );

        return $data;
    }

    /**
     * Get the URL where to redirect the customer to pay for the order
     *
     * @param array $paymentData The Payment Data from self::preparePaymentData
     *
     * @return string            The URL where to redirect the customer
     */
    public static function getRedirectUrl($paymentData)
    {
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioAPI.php';
        $api = new MontonioAPI(
            Configuration::get('MONTONIO_ACCESS_KEY'),
            Configuration::get('MONTONIO_SECRET_KEY'),
            Configuration::get('MONTONIO_ENVIRONMENT')
        );

        // get application draft
        $draftResult = $api->post_montonio_application_draft(json_encode($paymentData));

        // handle draft request response
        $accessToken = ($draftResult['status'] === 'SUCCESS') ? $draftResult['data']->access_token : null;

        $url   = self::getApplicationUrl();
        $query = parse_url($url, PHP_URL_QUERY);

        if ($query) {
            $url .= '&access_token=' . $accessToken;
        } else {
            $url .= '?access_token=' . $accessToken;
        }

        return $url;
    }

    /**
     * Get Montonio Financing application URL for current environment
     *
     * @return void
     */
    protected static function getApplicationUrl()
    {
        $env = Configuration::get('MONTONIO_ENVIRONMENT');

        return $env === 'sandbox' ? 'https://sandbox-application.montonio.com' : 'https://application.montonio.com';
    }

    // =========================================================================
    // Callback
    // =========================================================================

    public static function didUserCancel()
    {
        return false; // Not supported by Montonio Financing
    }

    /**
     * Get response from Montonio Financing
     *
     * @return array | null
     */
    public static function getCallbackResponse()
    {

        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioAPI.php';
        $api = new MontonioAPI(
            Configuration::get('MONTONIO_ACCESS_KEY'),
            Configuration::get('MONTONIO_SECRET_KEY'),
            Configuration::get('MONTONIO_ENVIRONMENT')
        );

        /**
         * Get Montonio Application from the API
         */
        $request = $api->get_montonio_application(Tools::getValue('id_cart'));

        /**
         * Exit if HTTP request to Montonio failed
         */
        if ($request['status'] !== 'SUCCESS') {
            return null;
        }

        $response = $request['data'];

        return array(
            'amount'             => $response->loan_amount,
            'is_paid'            => $response->status === 'signed',
            'merchant_reference' => $response->merchant_reference,
            'currency'           => $response->currency_code,
        );
    }

    /**
     * Get smarty data for legacy checkout option (Prestashop 1.6)
     *
     * @param array$params The params from hookPayment hook
     * @return array       The smarty params
     */
    public static function getLegacyPaymentOption($module, $params)
    {

        /**
         * Check if cart total is between the configured min and max
         */
        $min   = Configuration::get('MONTONIO_FINANCING_MIN_ORDER_AMOUNT');
        $max   = Configuration::get('MONTONIO_FINANCING_MAX_ORDER_AMOUNT');
        $total = $params['cart']->getOrderTotal();

        $min   = is_numeric($min) ? (float) $min : (float)self::MIN_ORDER_TOTAL;
        $max   = is_numeric($max) ? (float) $max : (float)self::MAX_ORDER_TOTAL;
        $total = is_numeric($total) ? (float) $total : null;

        $limitCheck = true;
        if (is_float($total)) {
            $limitCheck = $min <= $total && $total <= $max;
        }

        $enabled = self::isEnabled() && $module->isCurrencySupported(self::METHOD_CODE) && $limitCheck;

        return array(
            'enabled'     => $enabled,
            'title'       => $module->montonioFinancingTitle,
            'description' => $module->montonioFinancingDesc,
            'logo_url'    => self::getLogoUrl(),
            'show_logo'   => Configuration::get('MONTONIO_FINANCING_SHOW_LOGO') == '1',
        );
    }

    /**
     *
     * @return string
     */
    public static function getLogoUrl()
    {
        $logos = array(
            'et' => 'https://public.montonio.com/logo/financing/ee-financing.png',
            'lv' => 'https://public.montonio.com/logo/financing/lt-financing.png',
            'lt' => 'https://public.montonio.com/logo/financing/lv-financing.png',
            'en' => 'https://public.montonio.com/logo/financing/en-financing.png'
        );
        $lang = Context::getContext()->language->iso_code;
        return isset($logos[$lang]) ? $logos[$lang] : $logos['en'];
    }
}
