<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioOrderPrefixer.php';
require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';

class MontonioPayments
{
    const METHOD_CODE = 'MontonioPayments';
    const DEFAULT_LOGO_URL = 'https://public.montonio.com/logo/montonio-logomark-s.png';
    const SDK_PATH = 'montonio/vendor/MontonioPaymentsSDK.php';
    const AVAILABLE_LOCALES = array('de'=> 'de', 'en'=> 'en', 'et'=> 'et', 'fi'=> 'fi', 'lt'=> 'lt', 'lv'=> 'lv', 'pl'=> 'pl', 'ru'=> 'ru');

    public static function getConfig()
    {
        $options = array(
            array(
                'type'    => 'switch',
                'label'   => 'Enable Montonio Payments',
                'desc'    => 'When enabled, show Montonio Payments as a payment option at checkout',
                'is_bool' => true,
                'name'    => 'MONTONIO_PAYMENTS_ENABLED',
                'values'  => array(
                    array(
                        'id'    => 'active_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'active_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'    => 'switch',
                'label'   => 'Show logo?',
                'desc'    => 'You can turn this off if you don\'t want to show Montonio logo at checkout',
                'is_bool' => true,
                'name'    => 'MONTONIO_PAYMENTS_SHOW_LOGO',
                'values'  => array(
                    array(
                        'id'    => 'active_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'active_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'     => 'select',
                'label'    => 'Default Country',
                'desc'     => 'The country whose banks to show first at checkout.',
                'name'     => 'MONTONIO_PAYMENTS_DEFAULT_COUNTRY',
                'required' => 'true',
                'options'  => array(
                    'query' => array(
                        array(
                            'id'   => 'EE',
                            'name' => 'Estonia',
                        ),
                        array(
                            'id'   => 'FI',
                            'name' => 'Finland',
                        ),
                        array(
                            'id'   => 'LV',
                            'name' => 'Latvia',
                        ),
                        array(
                            'id'   => 'LT',
                            'name' => 'Lithuania',
                        ),
                        array(
                            'id' => 'PL',
                            'name' => 'Poland',
                        ),
                        array(
                            'id' => 'DE',
                            'name' => 'Germany',
                        ),
                    ),
                    'id'    => 'id',
                    'name'  => 'name',
                ),
            ),
            array(
                'type'    => 'switch',
                'label'   => 'Hide Country Select',
                'desc'    => 'You can turn this off if you don\'t want to show country select element on checkout, default or by address is used as fallback',
                'is_bool' => true,
                'name'    => 'MONTONIO_PAYMENTS_HIDE_COUNTRY',
                'values'  => array(
                    array(
                        'id'    => 'active_on',
                        'value' => 1,
                        'label' => 'Yes',
                    ),
                    array(
                        'id'    => 'active_off',
                        'value' => 0,
                        'label' => 'No',
                    ),
                ),
            ),
            array(
                'type'     => 'select',
                'label'    => 'Country by checkout data',
                'desc'     => '<strong>[ADVANCED]</strong> Should we attempt to change the selected country
                by the customer\'s info? If unsuccessful, we revert to your configured Default Country.',
                'name'     => 'MONTONIO_PAYMENTS_AUTOMATICALLY_CHANGE_COUNTRY',
                'required' => 'true',
                'options'  => array(
                    'query' => array(
                        array(
                            'id'   => 'manual',
                            'name' => 'No, show the configured Default Country',
                        ),
                        array(
                            'id'   => 'byLocaleIso',
                            'name' => 'Select country by locale',
                        ),
                    ),
                    'id'    => 'id',
                    'name'  => 'name',
                ),
            ),
            array(
                'type'     => 'select',
                'label'    => 'Country Dropdown Language',
                'desc'     => '<strong>[ADVANCED]</strong> How should the countries in the checkout dropdown be shown?',
                'name'     => 'MONTONIO_PAYMENTS_TRANSLATE_COUNTRY_DROPDOWN',
                'required' => 'true',
                'options'  => array(
                    'query' => array(
                        array(
                            'id'   => 'english',
                            'name' => 'in English',
                        ),
                        array(
                            'id'   => 'translated',
                            'name' => 'Translated',
                        ),
                    ),
                    'id'    => 'id',
                    'name'  => 'name',
                ),
            ),
            array(
                'type'         => 'textarea',
                'label'        => 'Custom CSS for Checkout page',
                'name'         => 'MONTONIO_PAYMENTS_PAYMENT_HANDLE_CSS',
                'autoload_rte' => false,
            ),
        );

        return $options;
    }

    const CONFIGDEFAULTS = array(
        'MONTONIO_PAYMENTS_ENABLED'                       => false,
        'MONTONIO_PAYMENTS_CREATE_ORDER'                  => 'before',
        'MONTONIO_PAYMENTS_SHOW_LOGO'                     => '1',
        'MONTONIO_PAYMENTS_HIDE_COUNTRY'                  => '0',
        'MONTONIO_PAYMENTS_STYLE'                         => 'grid_logos',
        'MONTONIO_PAYMENTS_DEFAULT_COUNTRY'               => 'EE',
        'MONTONIO_PAYMENTS_PAYMENT_HANDLE_CSS'            => '',
        'MONTONIO_PAYMENTS_AUTOMATICALLY_CHANGE_COUNTRY'  => 'manual',
        'MONTONIO_PAYMENTS_AUTOMATICALLY_SELECT_METHOD' => 'no',
        'MONTONIO_PAYMENTS_TRANSLATE_COUNTRY_DROPDOWN'    => 'english',
    );

    const TRANSLATIONS = array(
        'title'       => 'Pay with your bank',
        'description' => 'Complete the purchase with a direct payment from your bank account.',
    );

    /**
     * @return bool
     */
    public static function isEnabled()
    {
        return Configuration::get('MONTONIO_PAYMENTS_ENABLED') === '1';
    }

    /**
     * @param $module
     * @return PaymentOption|null
     */
    public static function getCheckoutPaymentOption($module)
    {
        /**
         * Check if option is enabled and supports selected currency
         */
        if (
            !self::isEnabled() ||
            !$module->isCurrencySupported(self::METHOD_CODE)
        ) {
            return null;
        }

        /**
         * Form action URL. The form data will be sent to the
         * validation controller when the user presses Place Order
         */
        $formAction = Context::getContext()->link->getModuleLink(
            $module->name,
            'validation',
            ['method' => self::METHOD_CODE],
            true
        );

        /**
         * Assign the url form action to the template var $action
         */
        Context::getContext()->smarty->assign(array(
            'action' => $formAction,
            'montonioPaymentsPaymentHandleCSS' => Configuration::get('MONTONIO_PAYMENTS_PAYMENT_HANDLE_CSS'),
        ));

        /**
         * Get Description for Payment method
         */
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsCheckout.php';
        $checkout = new Montonio_Payments_Checkout();
        $checkout->set_description($module->montonioPaymentsDesc);

        /**
         * Set the initial country for dropdown
         */
        $defaultCountry = Configuration::get('MONTONIO_PAYMENTS_DEFAULT_COUNTRY');
        if (empty($defaultCountry)) {
            $defaultCountry = 'EE';
        }

        $optionChangeCountryMode = Configuration::get('MONTONIO_PAYMENTS_AUTOMATICALLY_CHANGE_COUNTRY');
        if ($optionChangeCountryMode == 'byLocaleIso') {
            $customerLanguage = Language::getIsoById((int) Context::getContext()->cookie->id_lang);
            // TODO: Check if not exists
            $regionsByLanguage = array(
                'et' => 'EE',
                'fi' => 'FI',
                'lv' => 'LV',
                'lt' => 'LT',
                'pl' => 'PL',
                'de' => 'DE'
            );
            foreach ($regionsByLanguage as $locale => $countryCode) {
                if ($locale == $customerLanguage) {
                    $defaultCountry = $countryCode;
                    break;
                }
            }
        }

        $checkout->set_preferred_country($defaultCountry);

        /**
         * Translate the dropdown if needed
         */
        $optionTranslateDropdown = Configuration::get('MONTONIO_PAYMENTS_TRANSLATE_COUNTRY_DROPDOWN');
        if ($optionTranslateDropdown == 'translated') {
            $checkout->set_regions(array(
                'EE' => $module->textRegionEE,
                'LT' => $module->textRegionLT,
                'LV' => $module->textRegionLV,
                'FI' => $module->textRegionFI,
                'PL' => $module->textRegionPL,
                'DE' => $module->textRegionDE
            ));
        }

        $checkout->set_payment_handle_style(Configuration::get('MONTONIO_PAYMENTS_STYLE'));

        $bankList = (Configuration::get('MONTONIO_ENVIRONMENT') == 'sandbox')
        ? 'MONTONIO_PAYMENTS_CURRENT_BANKS_SANDBOX'
        : 'MONTONIO_PAYMENTS_CURRENT_BANKS';

        $checkout->set_banklist(self::getAvailableMethods($bankList));

        $checkout->set_attempt_to_select_payment(
            Configuration::get('MONTONIO_PAYMENTS_AUTOMATICALLY_SELECT_METHOD', 'no')
        );

        $description = $checkout->get_description_html();

        $title = $module->montonioPaymentsTitle;

        /**
         *  Load form template to be displayed in the checkout step
         */
        $paymentForm = $module->fetch('module:montonio/views/templates/hook/montonio_payments_payment_options.tpl');

        /**
         * Create a PaymentOption object containing the necessary data
         * to display this module in the checkout
         */
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
        $newOption->setModuleName($module->displayName)
            ->setCallToActionText($title)
            ->setAdditionalInformation($description)
            ->setAction($formAction)
            ->setForm($paymentForm);

        /**
         * Get Logo for Payment method and
         * assign the description to the template var $logo
         */
        $logo = Configuration::get('MONTONIO_PAYMENTS_SHOW_LOGO');
        if ($logo != '0') {
            $newOption->setLogo(self::getLogoUrl());
        }

        return $newOption;
    }

    /**
     * Get a list of available payment methods from API
     *
     * @param string $configKey
     * @return string
     */
    protected static function getAvailableMethods($configKey)
    {
        require_once _PS_MODULE_DIR_ . self::SDK_PATH;

        $ttl = !empty(Configuration::get($configKey . '_TTL')) ? Configuration::get($configKey . '_TTL') : false;

        $time = time();
        $extra = 24 * 60 * 60;

        if(!$ttl || $ttl < $time) {
            $sdk = new MontonioPaymentsSDK(
                Configuration::get('MONTONIO_ACCESS_KEY'),
                Configuration::get('MONTONIO_SECRET_KEY'),
                Configuration::get('MONTONIO_ENVIRONMENT')
            );

            $methods = $sdk->fetch_payment_methods();
            $result = isset($methods['data']) ? $methods['data'] : [];

            Configuration::updateValue($configKey. '_TTL', $time + $extra);
            Configuration::updateValue($configKey, json_encode($result));
        }

        return Configuration::get($configKey);
    }

    /**
     * Prepare the data to be submitted to Montonio Payments
     *
     * @param object $context The execution context
     * @param object $module Montonio class instance
     *
     * @return array          The data that will be sent to Montonio
     */
    public static function preparePaymentData($context, $module)
    {

        $cart = $context->cart;
        $customer = $context->customer;

        // add Prefix to Cart ID if needed
        $storePrefix = Configuration::get('MONTONIO_ORDER_PREFIX');
        $cartId = (string)$cart->id;

        $notificationUrlArgs = array(
            'method'          => self::METHOD_CODE,
            'id_cart'         => $cartId,
            'id_module'       => $module->id,
            'key'             => $customer->secure_key,
            'is_notification' => 'yes',
        );

        $returnUrlArgs = array(
            'method'    => self::METHOD_CODE,
            'id_cart'   => $cartId,
            'id_module' => $module->id,
            'key'       => $customer->secure_key,
        );

        // Shipping modules' Integration ********************************************************
        $carrier = new Carrier($cart->id_carrier);
        $carrierInfo = null;
        $carrierName = $carrier->external_module_name;
        if ($carrierName) {
            $key = $carrierName . '_' . $cart->id_address_delivery;
            if (isset($context->cookie->$key)) {
                $carrierInfo = $context->cookie->$key;
            }
        }

        if ($carrierInfo) {
            $notificationUrlArgs['extra_data'] = array($key => $carrierInfo);
            $returnUrlArgs['extra_data']       = array($key => $carrierInfo);
        }
        // /Shipping modules' Integration *******************************************************

        $moduleName = $module->name;

        /**
         * Generate order before redirect
         */
        $createOrderBefore = true;
        if (Configuration::get('MONTONIO_PAYMENTS_CREATE_ORDER') === 'before') {
            $createOrderBefore = true;
        }
        if ($createOrderBefore) {

            //order already exist no need to try and create it again but we need to throw to understand
            if (!Validate::isLoadedObject($context->cart) || $context->cart->OrderExists() === true) {
                throw new PrestaShopException('Cart cannot be loaded or an order has already been placed using this cart');
            }

            $module->validateOrder(
                (int)$cart->id,
                Configuration::get('PS_OS_MONTONIO'),
                $cart->getOrderTotal(),
                $module->displayName . ' (' .self::METHOD_CODE . ', Cart ID: ' . (int)$cart->id . ')',
                'Order Created with ' . self::METHOD_CODE,
                NULL,
                (int)$context->currency->id,
                false,
                $context->customer->secure_key
            );
        }

        if (isset($module->currentOrder)) {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $module->currentOrder);
            $order = new Order((int)$module->currentOrder);
        } else {
            $orderRef = MontonioOrderPrefixer::addPrefix($storePrefix, $cartId);
            $order = $context->cart;
        }

        $merchantRef = $orderRef . ((Configuration::get('MONTONIO_ENVIRONMENT') === 'sandbox') ? '_' . rand() : '');

        //can change on sandbox
        $returnUrlArgs['id_cart'] = $merchantRef;
        $notificationUrlArgs['id_cart'] = $merchantRef;

        $idCurrency = $order->id_currency;
        $paymentCurrency = new Currency($idCurrency); // should contain iso_code
        $currencyIsoCode = $paymentCurrency->iso_code;

        $orderTotal = isset($order->total_paid) ? $order->total_paid : $order->getOrderTotal();

        $langId = Language::getIsoById((int)Context::getContext()->cookie->id_lang);
        $availableLocales = self::AVAILABLE_LOCALES;

        $config = array(
            'merchantReference' => $merchantRef,
            'returnUrl' => $context->link->getModuleLink($moduleName, 'callback', $returnUrlArgs),
            'notificationUrl' => $context->link->getModuleLink($moduleName, 'callback', $notificationUrlArgs),
            'currency' => $currencyIsoCode,
            'grandTotal' => (float)$orderTotal,
            'locale' => (isset($availableLocales[$langId])) ? $availableLocales[$langId] : 'en',
        );

        return self::getPayload($config, $order, $module, $context);
    }

    /**
     * Prepare data transfer object
     *
     * @param array $config
     * @param Cart|Order $order
     * @param object $module
     * @return array
    */
    public static function getPayload(array $config, $order, $module, $context)
    {
        $payload = $config;

        if($context->customer->isLogged()) {
            $customer = new Customer((int) $order->id_customer);
        } else {
            //order object does not have id_guest , cart has
            $id_guest = isset($order->id_guest) ? $order->id_guest : $order->id_customer;
            $customer = new Customer((int)$id_guest);
        }

        $shippingAddress = new Address((int)$order->id_address_delivery);
        $payload['shippingAddress'] = self::getAddressPayload($shippingAddress, $customer->email);

        $billingAddress = new Address((int)$order->id_address_invoice);
        $payload['billingAddress'] = self::getAddressPayload($billingAddress, $customer->email);

        foreach($order->getProducts() as $product) {
            $name = isset($product['name']) ? $product['name'] : $product['product_name'];
            $quantity = isset($product['quantity']) ? $product['quantity'] : $product['product_quantity'];
            $payload['lineItems'][] = [
                'name'       => (string)Tools::safeOutput($name),
                'quantity'   => (int)Tools::safeOutput($quantity),
                'finalPrice' => (float)Tools::safeOutput($product['total_wt']),
            ];
        }

        $defaultCountry = Configuration::get('MONTONIO_PAYMENTS_DEFAULT_COUNTRY');
        if (empty($defaultCountry)) {
            $defaultCountry = 'EE';
        }
        $optionChangeCountryMode = Configuration::get('MONTONIO_PAYMENTS_AUTOMATICALLY_CHANGE_COUNTRY');
        if ($optionChangeCountryMode == 'byLocaleIso') {
            $customerLanguage = Language::getIsoById((int) Context::getContext()->cookie->id_lang);
            $regionsByLanguage = array(
                'et' => 'EE',
                'fi' => 'FI',
                'lv' => 'LV',
                'lt' => 'LT',
                'pl' => 'PL',
                'de' => 'DE'
            );
            foreach ($regionsByLanguage as $locale => $countryCode) {
                if ($locale == $customerLanguage) {
                    $defaultCountry = $countryCode;
                    break;
                }
            }
        }

        if (Tools::getValue('preselectedCountry') && Tools::getValue('preselectedCountry') != '') {
            $defaultCountry = Tools::getValue('preselectedCountry');
        }

        $payload['payment'] = array(
            'method' => 'paymentInitiation',
            'methodDisplay' => (string)Tools::safeOutput($module->montonioPaymentsTitle),
            'methodOptions' => array(
                // paymentReference
                // Structured payment reference number.
                // This is a standardised reference number used for accounting purposes and will be validated by banks.
                // Do not include it in the data if you do not use reference numbers to link payments.
                //'paymentReference' => '',
                'paymentDescription' => (string)Tools::safeOutput('Payment for order '. $payload['merchantReference']),
                'preferredCountry' => $defaultCountry,
                'preferredLocale' => $payload['locale'],
                'preferredProvider' => (Tools::getValue('montonio_payments_preselected_aspsp')) ? Tools::getValue('montonio_payments_preselected_aspsp') : ''
            ),
            'amount' => $payload['grandTotal'],
            'currency' => $payload['currency']
        );
        return $payload;
    }

    /**
     * Get the URL where to redirect the customer to pay for the order
     *
     * @param array $paymentData The Payment Data from self::preparePaymentData
     *
     * @return string            The URL where to redirect the customer
     */
    public static function getRedirectUrl($paymentData)
    {

        require_once _PS_MODULE_DIR_ . self::SDK_PATH;
        $sdk = new MontonioPaymentsSDK(
            Configuration::get('MONTONIO_ACCESS_KEY'),
            Configuration::get('MONTONIO_SECRET_KEY'),
            Configuration::get('MONTONIO_ENVIRONMENT')
        );

        $sdk->setPaymentData($paymentData);

        return $sdk->getPaymentUrl();
    }

    // =========================================================================
    // Callback
    // =========================================================================
    public static function didUserCancel()
    {
        return !Tools::getValue('order-token');
    }

    public static function getCallbackResponse()
    {
        $token = Tools::getValue('order-token');
        require_once _PS_MODULE_DIR_ . self::SDK_PATH;

        $result = false;

        try {
            $response = MontonioPaymentsSDK::decodePaymentToken($token, Configuration::get('MONTONIO_SECRET_KEY'));
            $result = array(
                'amount' => $response->grandTotal,
                'is_paid' => $response->payment_status === 'PAID',
                'access_key' => $response->accessKey,
                'merchant_reference' => $response->merchantReference
            );

            if (isset($response->payment_method_name)) {
                $result['payment_method_name'] = $response->payment_method_name;
            }

        } catch (Exception $e) {
        }
        return $result;
    }


    /**
     * Get payload address structured data from address object
     *
     * @param Address $address
     * @param $email
     * @return array
     */
    public static function getAddressPayload(Address $address, $email)
    {
        $data = array(
            'firstName' => (string)Tools::safeOutput($address->firstname),
            'lastName' => (string)Tools::safeOutput($address->lastname),
            'email' => (string)Tools::safeOutput($email),
            'addressLine1' => (string)Tools::safeOutput($address->address1),
            'addressLine2' => (string)Tools::safeOutput(isset($address->address2) ? $address->address2 : ''),
            'locality' => (string)Tools::safeOutput($address->city),
            'region' => (string)Tools::safeOutput(State::getNameById($address->id_state)),
            'postalCode' => (string)Tools::safeOutput($address->postcode),
            'phoneNumber' => (string)Tools::safeOutput((isset($address->phone_mobile) ? $address->phone_mobile : $address->phone)),
        );

        if (isset($address->id_country)) {
            $addressCountry = new Country($address->id_country);
            $data['country'] = (string)Tools::safeOutput($addressCountry->iso_code);
        }

        $data = array_filter($data);
        return $data;
    }

    /**
     * Get smarty data for legacy checkout option (Prestashop 1.6)
     *
     * @param array$params The params from hookPayment hook
     * @return array       The smarty params
     */
    public static function getLegacyPaymentOption($module)
    {
        $enabled = self::isEnabled() && $module->isCurrencySupported('MontonioPayments');

        /**
         * Get Description for Payment method
         */
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsCheckout.php';
        $checkout = new Montonio_Payments_Checkout();
        $checkout->set_description($module->montonioPaymentsDesc);
        $checkout->setPaymentLinkBase(Context::getContext()->link->getModuleLink('montonio', 'validation', array(
            'method' => 'MontonioPayments',
        )));

        /**
         * Set the initial country for dropdown
         */
        $defaultCountry = Configuration::get('MONTONIO_PAYMENTS_DEFAULT_COUNTRY');
        if (empty($defaultCountry)) {
            $defaultCountry = 'EE';
        }

        $optionChangeCountryMode = Configuration::get('MONTONIO_PAYMENTS_AUTOMATICALLY_CHANGE_COUNTRY');
        if ($optionChangeCountryMode == 'byLocaleIso') {
            $context = Context::getContext();
            $customerLanguage = Language::getIsoById((int) $context->cookie->id_lang);
            // TODO: Check if not exists
            $regionsByLanguage = array(
                'et' => 'EE',
                'fi' => 'FI',
                'lv' => 'LV',
                'lt' => 'LT',
                'pl' => 'PL',
                'de' => 'DE'
            );
            foreach ($regionsByLanguage as $locale => $countryCode) {
                if ($locale == $customerLanguage) {
                    $defaultCountry = $countryCode;
                    break;
                }
            }
        }

        $checkout->set_preferred_country($defaultCountry);

        /**
         * Translate the dropdown if needed
         */
        $optionTranslateDropdown = Configuration::get('MONTONIO_PAYMENTS_TRANSLATE_COUNTRY_DROPDOWN');
        if ($optionTranslateDropdown == 'translated') {
            $checkout->set_regions(array(
                'EE' => $module->textRegionEE,
                'LT' => $module->textRegionLT,
                'LV' => $module->textRegionLV,
                'FI' => $module->textRegionFI,
                'PL' => $module->textRegionPL,
                'DE' => $module->textRegionDE
            ));
        }

        $checkout->set_payment_handle_style(Configuration::get('MONTONIO_PAYMENTS_STYLE'));
        $checkout->set_payment_handle_extra_css(''); // TODO

        $bankList = (Configuration::get('MONTONIO_ENVIRONMENT') == 'sandbox')
        ? 'MONTONIO_PAYMENTS_CURRENT_BANKS_SANDBOX'
        : 'MONTONIO_PAYMENTS_CURRENT_BANKS';

        $checkout->set_banklist(self::getAvailableMethods($bankList));
        $banksUI = $checkout->get_description_html();

        $paymentHandleCss = Configuration::get('MONTONIO_PAYMENTS_PAYMENT_HANDLE_CSS');

        $montonioPaymentsData = array(
            'enabled'            => $enabled,
            'title'              => 'MONTONIO PAYMENTS',
            'description'        => Configuration::get('MONTONIO_PAYMENTS_DESCRIPTION'),
            'logo_url'           => self::getLogoUrl(),
            'show_logo'          => Configuration::get('MONTONIO_PAYMENTS_SHOW_LOGO') == '1',
            'paymentHandleStyle' => Configuration::get('MONTONIO_PAYMENTS_STYLE'),
            'showCountry'        => Configuration::get('MONTONIO_PAYMENTS_HIDE_COUNTRY'),
            'banksUI'            => $banksUI,
        );

        if ($paymentHandleCss != '' && !empty($paymentHandleCss)) {
            $montonioPaymentsData['paymentHandleCss'] = $paymentHandleCss;
        }

        return $montonioPaymentsData;
    }

    /**
     *
     * @return string
     */
    public static function getLogoUrl()
    {
        return self::DEFAULT_LOGO_URL;
    }
}
